local lt = CreateFrame("Frame")
local Begin = CreateFrame("Frame");
local Leveled = CreateFrame("Frame");

local playerName = UnitName("player")
local realmName = GetRealmName()
local _LT
local logging = false

-- sloppiest and most inefficient code you'll ever see

do
   local requesting
   local o = _G.ChatFrame_DisplayTimePlayed
   _G.ChatFrame_DisplayTimePlayed = function(...)
      if requesting then
         requesting = false
         return
      end
      return o(...)
   end
   function lt:RequestTimePlayed()
      requesting = true
      _G.RequestTimePlayed()
   end
end 

local function maxLevel()
	local maxlvl = MAX_PLAYER_LEVEL_TABLE[GetExpansionLevel()]
	if UnitLevel("player") == maxlvl then
		logging = false
		return true
	else
		return false
	end
end

local function parseTime(seconds)
	local seconds = tonumber(seconds)
	hours = string.format("%01.f", math.floor(seconds/3600));
	mins = string.format("%01.f", math.floor(seconds/60 - (hours*60)));
	secs = string.format("%01.f", math.floor(seconds - (hours*3600) - (mins *60)));
	return hours.." hours, "..mins.." minutes, "..secs.." seconds"
end

local function highestlogged()
   local levels = {}
   for key, item in pairs(_LT) do
      tinsert(levels, key)
   end
   return tonumber(math.max(unpack(levels)) + 1)
end

local function lowestlogged()
   local levels = {}
   for key, item in pairs(_LT) do
      tinsert(levels, key)
   end
   return tonumber(math.min(unpack(levels)) + 1)
end

local function timeString(...)
	local lvl1, lvl2
	lvl1 = select(1, ...)
	if select(2, ...) then 
		lvl2 = select(2, ...)
	end
	local str
	if lvl2 == nil then	
		str = "It took " .. parseTime(_LT[lvl1 - 1]["total"]) .. " from level " .. lvl1 - 1 .. " to level " .. lvl1 .. "."
	else
		local totalSec = 0
		for key, item in pairs(_LT) do
			if key >= lvl1 and key < lvl2 then
				totalSec = totalSec + tonumber(_LT[key]["total"])
			end
		end
		str = "It took " .. parseTime(totalSec) .. " from level " .. lvl1 .. " to level " .. lvl2 .. "."
	end
	return str
end

local function LevelTimerSlash(msg, editbox)
	if msg == 'help' then
		print('LevelTimer: Available Commands:')
		print("LevelTimer: '/lt 3' Returns the leveling time for a specific level.")
		print("LevelTimer: '/lt 20-30' Returns the leveling time for the specified level range.")		
		return
	end	
	if next(_LT) == nil and tonumber(msg) ~= nil then
		print ("LevelTimer: You don't have any data logged yet.")
		return
	end
	if tonumber(msg) ~= nil then
		if tonumber(msg) >= lowestlogged() and tonumber(msg) <= highestlogged() then
			print(timeString(msg))
			return
		else
			print ("LevelTimer: That level has not been logged.")
			return
		end
	end
	msg = msg:gsub("%s+", "")
	local _,_,num1,num2 = string.find(msg, "(%w+)-(%w+)")
	local range = {}

	if tonumber(num1) == nil or tonumber(num2) == nil then
	   print("LevelTimer: Syntax error, type '/lt help' for a list of commands.")
	   return
	end	
	tinsert(range, num1)
	tinsert(range, num2)
	local min = tonumber(math.min(unpack(range)))
	local max = tonumber(math.max(unpack(range)))
	
	if next(_LT) == nil then
		print ("LevelTimer: You don't have any data logged yet.")
		return
	end 
	
	if min < lowestlogged()-1 then
		print ("LevelTimer: Data for level " .. min .. " is not logged.")
		return
	end
	if max > highestlogged() then
		print ("LevelTimer: Data for level " .. max .. " is not logged.")
		return
	end
	print(timeString(min, max))
end

SLASH_LEVELTIMER1, SLASH_LEVELTIMER2 = '/lt', '/leveltimer'
SlashCmdList["LEVELTIMER"] = LevelTimerSlash 

local function beginLogging(self, event, arg1, arg2, ...)
	if _LT[UnitLevel("player")] == nil and not maxLevel() then
		print ("LevelTimer: Beginning to log leveling time for " .. UnitName("player") .. ".")
		_LT[UnitLevel("player")] = {}
		_LT[UnitLevel("player")]["begin"] = arg1
		Begin:UnregisterEvent("TIME_PLAYED_MSG")
	end
end

local function playerLeveledUp(self, event, arg1, arg2, ...)
	_LT[UnitLevel("player") -1]["end"] = arg1
	_LT[UnitLevel("player") -1]["total"] = _LT[UnitLevel("player")-1]["end"] - _LT[UnitLevel("player")-1]["begin"]
	if not maxLevel() then
		_LT[UnitLevel("player")] = {}
		_LT[UnitLevel("player")]["begin"] = arg1
	end
	print(timeString(UnitLevel("player")))
	Leveled:UnregisterEvent("TIME_PLAYED_MSG")
end

local function eventHandler(self, event, arg1, arg2, ...)
	if event == "PLAYER_LOGIN" then
		if type(LevelTimer) ~= "table" then LevelTimer = {} end
		if type(LevelTimer[realmName]) ~= "table" then LevelTimer[realmName] = {} end
		if type(LevelTimer[realmName][playerName]) ~= "table" then LevelTimer[realmName][playerName] = {} end
		_LT = LevelTimer[realmName][playerName]
	end	
	if event == "QUEST_LOG_UPDATE" then
		if not maxLevel() and not logging then
			logging = true
			Begin:RegisterEvent('TIME_PLAYED_MSG')
			Begin:SetScript('OnEvent', beginLogging)
			lt:RequestTimePlayed()
		end
	end
	if event == "PLAYER_XP_UPDATE" then
		if not maxLevel() and not logging then
			logging = true
			if _LT[UnitLevel("player")] == nil then
				Begin:RegisterEvent('TIME_PLAYED_MSG')
				Begin:SetScript('OnEvent', beginLogging)
				lt:RequestTimePlayed()
			end
		end
	end	
	if event == "PLAYER_LEVEL_CHANGED" then
		Leveled:RegisterEvent('TIME_PLAYED_MSG')
		Leveled:SetScript('OnEvent', playerLeveledUp)
		lt:RequestTimePlayed()
	end
end


lt:SetScript('OnEvent', eventHandler)
lt:RegisterEvent("PLAYER_LOGIN")
lt:RegisterEvent("PLAYER_LEVEL_CHANGED")
lt:RegisterEvent("PLAYER_XP_UPDATE")
lt:RegisterEvent("QUEST_LOG_UPDATE")

